// pascal statements parser.
// author: Dmitry Boyarintsev
unit PascalParsing;

{$mode delphi}

interface

uses
  Classes, SysUtils, ExprParse;

function ParseCodeBlock(parser: TExpParser): TStatement; overload;
function ParseCodeBlock(const s: AnsiString): TStatement; overload;

implementation

function ParseIfStatement(parser: TExpParser): TStatement;
var
  ifst : TIfStatement;
  i    : Integer;
begin
  // 'if' has already been parsed!
  Result:=nil;
  ifst := TIfStatement.Create;

  i:=parser.ExpEnd.Add('then');
  ifst.Condition:=ParseExpression(parser);
  parser.ExpEnd.Delete(i);
  if parser.Token<>'then' then begin
    ifst.Free;
    Exit;
  end;

  parser.NextToken;
  i:=parser.ExpEnd.Add('else');
  ifst.ThenStatement:=ParseCodeBlock(parser);
  parser.ExpEnd.Delete(i);

  if parser.Token = 'else' then begin
    parser.NextToken;
    ifst.ElseStatement:=ParseCodeBlock(parser);
  end;
  Result:=ifst;
end;

function ParseWhileStatement(parser: TExpParser): TStatement;
var
  wl  : TWhileStatement;
  i   : Integer;
begin
  Result:=nil;
  wl := TWhileStatement.Create;
  i:=parser.ExpEnd.Add('do');
  wl.Condition:=ParseExpression(parser);
  parser.ExpEnd.Delete(i);
  if parser.Token<>'do' then begin
    wl.Free;
    Exit;
  end;
  parser.NextToken;
  wl.Body:=ParseWhileStatement(parser);
end;

function ParseCodeBlock(parser: TExpParser): TStatement;
var
  first   : TStatement;
  last    : TSTatement;
  st      : TStatement;
  asst    : TAssignStatement;
  waitend : Boolean;

  left    : TExpression;
begin
  last:=nil;
  first:=nil;

  waitend:=parser.Token='begin';
  if waitend then parser.NextToken;

  repeat
    if parser.Token='if' then
      st:=ParseIFStatement(parser)
    else if parser.Token='while' then
      st:=ParseWhileStatement(parser)
    else begin
      parser.index:=parser.TokenPos;
      left:=ParseExpression(parser);
      if parser.Token=':=' then begin
        asst:=TAssignStatement.Create;
        asst.Left:=left;
        asst.Right:=ParseExpression(parser);
        st:=asst;
        parser.NextToken;
      end else if parser.Token=';' then begin
        parser.NextToken;
        st:=TExpressionStatement.Create;
        TExpressionStatement(st).Expression:=left;
      end;
    end;

    if Assigned(st) then begin
      if Assigned(last) then last.next:=st;
      if not Assigned(first) then first:=st;
      last:=st;
    end;
  until (not waitend) or (parser.Token='end') or (parser.Token = '') or (parser.Token='until');

  if (waitend) and (parser.Token='end') then parser.NextToken;
  Result:=first;
end;

function ParseCodeBlock(const s: AnsiString): TStatement;
var
  p : TExpParser;
begin
  p:=TExpParser.Create;
  p.Text:=s;
  p.Index:=1;
  p.NextToken;
  if p.Token<>'begin' then
    p.index:=p.TokenPos;
  Result:=ParseCodeBlock(p);
  p.Free;
end;

end.

