// expression parser
// author: Dmitry Boyarintsev
program project1;

{$mode objfpc}{$H+}

uses
  Classes, ExprParse, TextParsingUtils, PascalParsing;

procedure WriteExp(exp: TExpression; var s: String); forward;

procedure WriteBinExp(exp: TBinExp; var s: string);
var
  r, l  : String;
begin
  if Assigned(exp.left) then begin
    WriteExp(exp.left, l);
    l:='('+l+')'
  end else
    l:=exp.leftid;

  if Assigned(exp.right) then begin
    WriteExp(exp.right, r);
    r:='('+r+')'
  end else
    r:=exp.rightid;

  s:=l+exp.oper+r;
end;

procedure WriteExp(exp: TExpression; var s: String);
var
  t : String;
begin
  if not Assigned(exp) then Exit;
  if exp is TBinExp then WriteBinExp(exp as TBinExp, s);
  if exp is TIdentExp then s:=TIdentExp(exp).ident;
  if exp is TUnaryPrefixExp then begin
    if Assigned(TUnaryPrefixExp(exp).right) then
      WriteExp(TUnaryPrefixExp(exp).right, t)
    else
      t:=TUnaryPrefixExp(exp).rightid;
    s:=exp.oper+' '+t;
  end;
end;

procedure EvalBin(ex: TExpression; var res: Integer);
var
  r,  l : Integer;
  err   : Integer;
  exp   : TBinExp;
  u     : TUnaryPrefixExp;
begin
  if ex is TBinExp then begin
    exp:=ex as TBinExp;
    if Assigned(exp.left) then EvalBin(exp.left, l) else Val(exp.leftid, l, err);
    if Assigned(exp.right) then EvalBin(exp.right, r) else Val(exp.rightid, r, err);
    case exp.oper of
      '+': res:=l+r;
      '*': res:=l*r;
      '-': res:=l-r;
      'div': res:=l div r;
      'mod': res:=l mod r;
    end;
  end else if ex is TUnaryPrefixExp then begin
    u:=TUnaryPrefixExp(ex);
    if u.oper='not' then begin
      if Assigned(u.right) then EvalBin(u, r) else Val(u.rightid, r, err);
      res:=not r;
    end;
  end;
end;

procedure TestExpression;
var
  s   : string;
  r   : integer;
  exp : TExpression;
begin
  //s := '5 + not 2';
  //s:='b+c*2-(a+5)';
  s:='a+b*c*d*e+f+g';
  writeln(s);
  exp:=ParseExpression(s);
  writeln('res=');
  r:=0;
  if Assigned(exp) then begin
    s:='';
    WriteExp(exp, s);
    writeln(s);
    EvalBin(exp as TBinExp, r);
    //writeln('res = ', r);
  end else;
    //writeln('failed');
  writeln(3 + not 2);
end;


procedure WriteStatements(st: TStatement);
var
  s : String;
  l : String;
begin
  while Assigned(st) do begin
    if st is TIfStatement then begin
      WriteExp(TIfStatement(st).Condition, s);
      writeln('if (',s,') then');
      WriteStatements(TIfStatement(st).ThenStatement);
      if Assigned(TIfStatement(st).ElseStatement) then begin
        writeln('else: ');
        WriteStatements(TIfStatement(st).ElseStatement);
      end;
    end else if st is TExpressionStatement then begin
      WriteExp(TExpressionStatement(st).Expression, s);
      writeln(s);
    end else if st is TAssignStatement then begin
      WriteExp(TAssignStatement(st).Left, l);
      WriteExp(TAssignStatement(st).Right, s);
      writeln(l,':=',s);
    end;
    st:=st.Next;
  end;
end;

procedure TestStatement;
var
  st : TStatement;
begin
  st:=ParseCodeBlock(
     'begin'+
     ' if a then begin '+
     '   b:=1;'+
     '   c:=2;'+
     '   d:=b+c*2-(a+5);'+
     ' end else '+
     '   c; '+
     ' end;');
  writeln('st = ', PtrUInt(st));
  WriteStatements(st);
end;

function PasExpToJava(exp: TExpression): String; forward;

function UnaryPasExpToJava(exp: TUnaryPrefixExp): String;
begin
  if exp.oper='not' then begin
    if Assigned(exp.right) then
      Result:='~('+PasExpToJava(exp.right)+')'
    else
      Result:='~'+exp.rightid;
  end;
end;

function OpPasToJava(const op: String): String;
begin
  Result:=op;
  if Result='and' then Result:='&'
  else if Result='or' then Result:='|'
  else if Result='xor' then Result:='^'
  else if Result='shl' then Result:='<<'
  else if Result='shr' then Result:='>>'
  else if Result='div' then Result:='/'
  else if Result='mod' then Result:='%'
  else if Result='not' then Result:='~'
  else if Result='=' then Result:='=='
  else if Result='<>' then Result:='!=';
end;

function BinaryPasExpToJave(exp: TBinExp): String;
var
  l, r  : String;
begin
  if Assigned(exp.left) then l:='('+PasExpToJava(exp.left)+')' else l:=exp.leftid;
  if Assigned(exp.right) then r:='('+PasExpToJava(exp.right)+')' else r:=exp.rightid;
  Result:=l+OpPasToJava(exp.oper)+r;
end;

function PasExpToJava(exp: TExpression): String;
begin
  if exp is TUnaryPrefixExp then
    Result:=UnaryPasExpToJava(exp as TUnaryPrefixExp)
  else if exp is TBinExp then
    Result:=BinaryPasExpToJave(exp as TBinExp)
  else if exp is TIdentExp then
    Result:=TIdentExp(exp).ident
end;

procedure TestToJava;
var
  s   : AnsiString;
  exp : TExpression;
begin
  writeln('please type Pascal arithmetic/logical expression (without comments, please)');
  writeln('(the parser doesn''t see the differences between logical and bitwise operations)');
  writelN('i.e: a=b and 8-3');
  readln(s);
  exp:=ParseExpression(s);
  if Assigned(exp) then begin
    writeln(PasExpToJava(exp));
  end else
    writeln('sorry unable to parse!');
end;

begin
  //TestExpression;
  //TestStatement;
  TestToJava;
end.
